﻿using System;
using System.Windows.Forms;

using SharpDX.Direct2D1;
using SharpDX.Direct3D;
using SharpDX.Direct3D11;
using SharpDX.DXGI;
using SharpDX.Mathematics.Interop;
using SharpDX.WIC;

namespace TestProject
{
    /// <summary>
    /// 다이렉트 X 헬퍼
    /// </summary>
    public class DXHelper : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 팩토리 2
        /// </summary>
        private SharpDX.Direct2D1.Factory2 factory2 = null;

        /// <summary>
        /// 장치
        /// </summary>
        private SharpDX.Direct3D11.Device device = null;

        /// <summary>
        /// 스와프 체인
        /// </summary>
        private SwapChain swapChain = null;

        /// <summary>
        /// 렌더링 타겟
        /// </summary>
        private RenderTarget renderTarget = null;

        /// <summary>
        /// 비트맵
        /// </summary>
        private SharpDX.Direct2D1.Bitmap bitmap = null;

        /// <summary>
        /// 타겟 사각형
        /// </summary>
        private RawRectangleF targetRectangle;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 픽처 박스 초기화하기 - InitializePictureBox(pictureBox)

        /// <summary>
        /// 픽처 박스 초기화하기
        /// </summary>
        /// <param name="pictureBox">픽처 박스</param>
        public void InitializePictureBox(PictureBox pictureBox)
        {
            SwapChainDescription swapChainDescription = new SwapChainDescription()
            {
                BufferCount     = 1,
                ModeDescription = new ModeDescription
                (
                    pictureBox.Width,
                    pictureBox.Height,
                    new Rational(60, 1),
                    Format.R8G8B8A8_UNorm
                ),
                IsWindowed        = true,
                OutputHandle      = pictureBox.Handle,
                SampleDescription = new SampleDescription(1, 0),
                SwapEffect        = SwapEffect.Discard,
                Usage             = Usage.RenderTargetOutput
            };

            this.targetRectangle.Left   = 0;
            this.targetRectangle.Top    = 0;
            this.targetRectangle.Right  = pictureBox.Width;
            this.targetRectangle.Bottom = pictureBox.Height;

            SharpDX.Direct3D11.Device.CreateWithSwapChain
            (
                DriverType.Hardware,
                DeviceCreationFlags.BgraSupport,
                swapChainDescription,
                out this.device,
                out this.swapChain
            );

            this.factory2 = new SharpDX.Direct2D1.Factory2();

            Surface backSurface = Surface.FromSwapChain(this.swapChain, 0);

            RenderTargetProperties properties = new RenderTargetProperties
            (
                new SharpDX.Direct2D1.PixelFormat
                (
                    Format.Unknown,
                    SharpDX.Direct2D1.AlphaMode.Premultiplied
                )
            );

            this.renderTarget = new RenderTarget(this.factory2, backSurface, properties);
        }

        #endregion
        #region 비트맵 로드하기 - LoadBitmap(filePath)

        /// <summary>
        /// 비트맵 로드하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        public void LoadBitmap(string filePath)
        {
            ImagingFactory2 imagingFactory2 = new ImagingFactory2();

            BitmapDecoder bitmapDecoder = new BitmapDecoder(imagingFactory2, filePath, DecodeOptions.CacheOnDemand);

            BitmapFrameDecode bitmapFrameDecode = bitmapDecoder.GetFrame(0);

            FormatConverter formatConverter = new FormatConverter(imagingFactory2);

            formatConverter.Initialize(bitmapFrameDecode, SharpDX.WIC.PixelFormat.Format32bppPRGBA);

            this.bitmap = SharpDX.Direct2D1.Bitmap.FromWicBitmap(this.renderTarget, formatConverter);

            formatConverter.Dispose();

            bitmapFrameDecode.Dispose();

            bitmapDecoder.Dispose();

            imagingFactory2.Dispose();
        }

        #endregion
        #region 렌더링하기 - Render()

        /// <summary>
        /// 렌더링하기
        /// </summary>
        public void Render()
        {
            this.renderTarget.BeginDraw();

            this.renderTarget.Clear(new RawColor4(1.0f, 1.0f, 1.0f, 1.0f));

            this.renderTarget.DrawBitmap(this.bitmap, this.targetRectangle, 1.0f, SharpDX.Direct2D1.BitmapInterpolationMode.Linear);
          //this.renderTarget.DrawBitmap(this.bitmap, 1.0f, SharpDX.Direct2D1.BitmapInterpolationMode.Linear);

            this.renderTarget.EndDraw();

            this.swapChain.Present(1, PresentFlags.None);
        }

        #endregion
        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            this.renderTarget?.Dispose();

            this.renderTarget = null;

            this.swapChain?.Dispose();

            this.swapChain = null;
            
            this.device?.Dispose();

            this.device = null;

            this.factory2?.Dispose();

            this.factory2 = null;

            this.bitmap?.Dispose();

            this.bitmap = null;
        }

        #endregion
    }
}